from functools import partial
from charmhelpers.core import hookenv
from charmhelpers.core.services.base import ServiceManager
from charmhelpers.core.services import helpers
from charmhelpers.contrib.openstack.utils import os_release

import vpp_utils
import vpp_data


def manage():
    config = hookenv.config()
    release = os_release('nova-common')
    manager = ServiceManager([
        # Actions which have no prerequisites and can be rerun
        {
            'service': 'vpp-setup',
            'data_ready': [
                vpp_utils.install_packages,
            ],
            'provided_data': [
                vpp_data.NeutronPluginRelation(),
            ],
        },
        # Install hugepages and components reliant on huge pages
        {
            'service': 'vpp-compute-install',
            'required_data': [
                vpp_data.NeutronPluginRelation(),
            ],
            'data_ready': [
                vpp_utils.install_vpp_packages,
                vpp_utils.configure_ssh_keys,
            ],
        },
        {
            'service': 'vpp-compute-render',
            'required_data': [
                vpp_data.NeutronPluginRelation(),
                vpp_data.ODLControllerRelation(),
                config,
                vpp_data.ConfigTranslation(),
                vpp_data.PCIInfo(),
            ],
            'data_ready': [
                vpp_utils.bind_orphaned_net_interfaces,
                helpers.render_template(
                    source='qn.conf',
                    target='/opt/cisco/vpe/etc/qn.conf',
                    on_change_action=(partial(vpp_utils.restart_qn)),
                ),
                vpp_utils.odl_node_registration,
                vpp_utils.odl_register_macs,
                vpp_utils.bind_orphaned_net_interfaces,
                vpp_utils.odl_delete_stale_entries,
            ],
        },
    ])
    manager.manage()
